package cn.snowheart.dingtalk.robot.starter.autoconfiguration;

import cn.snowheart.dingtalk.robot.starter.client.DingTalkRobotClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.AutoConfigureOrder;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.client.RestTemplate;

/**
 * DingTalk机器人的AutoConfiguration，默认自动化加载一些配置。 这个类需要后加载
 *
 * @author Wanxiang Liu
 * @version 1.0.0
 */
@Configuration
@AutoConfigureOrder(Short.MAX_VALUE)
public class DingTalkRobotAutoConfiguration {

    /**
     * 钉钉机器人WebHook地址的默认前缀
     */
    @Value("${dingtalk.robot.prefix:https://oapi.dingtalk.com/robot/send}")
    private String urlPrefix;
    /**
     * 钉钉机器人WebHook地址的access_token
     */
    @Value("${dingtalk.robot.access-token:}")
    private String accessToken;

    /**
     * 钉钉机器人是否启用加签，默认false，启用加签需设置secret_token
     */
    @Value("${dingtalk.robot.secret.secret-enabled:false}")
    private boolean secretEnable;

    /**
     * 钉钉机器人WebHook地址的secret_token,群机器人加签用
     */
    @Value("${dingtalk.robot.secret.secret-token:null}")
    private String secretToken;

    /**
     * 创建一个RestTemplate客户端，并注册到Spring容器，供后续程序调用
     * @return RestTemplate客户端
     */
    @Bean(name = "dingTalkRobotRestTemplate")
    @ConditionalOnMissingBean
    public RestTemplate dingTalkRobotRestTemplate() {
        return new RestTemplate();
    }

    /**
     * 创建钉钉机器人本地调用客户端，并注册到Spring容器
     * @return 钉钉机器人本地调用客户端
     */
    @Bean
    @ConditionalOnMissingBean
    @ConditionalOnBean(name = "dingTalkRobotRestTemplate")
    public DingTalkRobotClient dingTalkRobotClient() {
        final DingTalkRobotClient dingTalkRobotClient = new DingTalkRobotClient();
        dingTalkRobotClient.setRestTemplate(dingTalkRobotRestTemplate());
        dingTalkRobotClient.setUrlPrefix(urlPrefix);
        dingTalkRobotClient.setSecretEnable(secretEnable);
        dingTalkRobotClient.setAccessToken(accessToken);
        dingTalkRobotClient.setSecretToken(secretToken);
        return dingTalkRobotClient;
    }
}